#!/usr/bin/perl
#
# ickle2kxicq.pl
# 
#  Perl script for converting ickle to kxicq contacts and history,
#  version 1.0 by mironcho. You will need Term::ANSIColor .
#  
#  Download:
#	http://mironcho.sourceforge.net
#
#  ToDo:
# 	Rebuild history converting. Code cleanup.
# 
#  Warranty:
#	NO WARRANTY, use at your risk and responsability!
#
#  Licence:
#	Released under the terms of the GNU General Public License as
#	published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#  Author:
#	Miroslav Yordanov <mironcholinux@mail.bg>
#	
use warnings;
use strict;
use File::Path;
use Term::ANSIColor qw(:constants);
$Term::ANSIColor::AUTORESET = 1;

sub history_get($);
sub time2mine($);

my $yellow = YELLOW;
my $red = RED;
my $green = GREEN;
my $reset = RESET;

# This MUST be allways 1 at startup 
my $msg_num = 1;

# Turn off system buffering to get working UI effects
$| = 1;

# Set $effects value to zero to turn off effects
my $effect = 1;


# Main::BEGIN ;)

# First, clearing screen
system ('clear');

print<<INFO;

           $red ickle2kxicq.pl $green
-----------------------------------------
Ickle to kxicq contacts convertor
version 1.0 by mironcho
Miroslav Yordanov $yellow<mironcholinux\@mail.bg> $green
 script site: $yellow http://mironcho.sourceforge.net $green
 ickle website: $yellow http://ickle.sourceforge.net $green
 kxicq website: $yellow http://www.kxicq.org $green
-----------------------------------------
INFO

# Well, I'm a STRICT boy, so .....
my ( @tmp, $uin, $home, $ickle_dir, $kxicq_dir, %contact, $nick, $i, $num);
$num = 0;


# Getting user HOME directory
if (!defined $ENV{'HOME'}) {
	print GREEN "Please enter the full path to your home directory: ";
	$home =  <STDIN> ;
	chomp $home;
} else {
	$home = $ENV{'HOME'};
}

# Delete ickle2kxicq temporary directory if exist from old sesseions
unlink "$home/.tmp/ickle2kxicq/history" if ( -e "$home/.tmp/ickle2kxicq/history");

$ickle_dir = "$home/.ickle/contacts";
$kxicq_dir = "$home/.kde/share/apps/kxicq2";

die RED "Directory $ickle_dir doesn't exist !!!" unless (-d $ickle_dir);
die RED "Directory $kxicq_dir doesn't exist !!!" unless (-d $kxicq_dir);

opendir IDIR, "$ickle_dir" or die RED "Could not open directory $ickle_dir: $!\n";
open KFILE, "> $kxicq_dir/contacts.kxicq" or die RED "Could not open/create contacts.kxicq: $!\n";

# Let's go ;)
print GREEN "Converting  ";
while ( $_ = readdir( IDIR ) ) {
	if ( $_ =~ /^[0-9]+\.user$/ ) {
		open I_FILE, "$ickle_dir/$_" or die RED "Could not open file $_: $!\n";

		# Checking every line for needed information.
		while ( defined (my $ickle_file = <I_FILE>) ) {
			chomp $ickle_file;
			@tmp = split /\s*=\s*/ , $ickle_file;
			($nick = $tmp[1]) if ( $tmp[0] eq "alias" );

			($uin = $tmp[1]) if ( $tmp[0] eq "uin" );
				

			
		}
		
		# If we have right values in this file, writing them ...
		if ((defined $nick) && (defined $uin)) {
			
			$contact{$uin} = $nick;

			( mkpath (["$home/.tmp/ickle2kxicq/"], 0, 0755) or die "Could not make temp dir: $!\n") 
				unless ( -d "$home/.tmp/ickle2kxicq/");

			die("Problem with history_get function: $!\n") unless (history_get($uin));
			
		    	print    ON_BLUE, " " , RESET;
			
			# Just looping a "little" for a nice UI effect 
			if ($effect) {
				for (1...70000){};
			}
			
			undef $uin;
			undef $nick;
			$num++ ;
		}

		close I_FILE;
	}
}


for (sort { $a <=> $b} (keys %contact)) {
    print KFILE "[Contact $_]\n";
    print KFILE "Authorized=true\n";
    print KFILE "Nick=$contact{$_}\n";
    print KFILE "UIN=$_\n\n";
}

# Now, writing the "[Default]" section
if ($num) {
	print KFILE "[Default]\nContacts=";
	
	$i=0;
	for my $curruin (keys %contact ) {
		$i++ && print KFILE ",";
		print KFILE $curruin;
	}

	print KFILE "\nCount=$num\n";

	# Now writing history file ..
	open HFILE,"> $home/.kde/share/apps/kxicq2/messages.kxicq2" or die "Could not open file: $!\n";
	open INHFILE, "$home/.tmp/ickle2kxicq/history" or die "Could not open file: $!\n";
	
	print HFILE "Messages=" . (scalar ($msg_num - 1)) . "\n";
	close HFILE;
	open HFILE, ">> $home/.kde/share/apps/kxicq2/messages.kxicq2" or die "Could not open file: $!\n";

	print HFILE while (<INHFILE>);
	
	close KFILE;
	close HFILE;
	close INHFILE;
	close IDIR;
	print "\n" , BLINK BOLD RED $num ,RESET, GREEN, "contacts are converted !!!\n\n";
}

# Turning on buffering and other prog_end stuffs ;)
$| = 0;

#
# CAUTION !!!!
# PLEASE DO NOT EDIT THIS LINE UNLESS YOU KNOW WHAT CAN HAPPEND !!!
# YOU CAN DELETE YOUR HOME DIRECTORY IF YOU CHANGE THIS UNCORRECTLY !!!
# OR IF YOU ARE ROOT - ALL LINUX TREE ('/')  !!!!!!!!!!!!!!!!!!!!!!!!
# SO, BE CAREFULL
# 
rmtree (["$home/.tmp/ickle2kxicq/"], 0, 1) if ( -d "$home/.tmp/ickle2kxicq/");



# Main::END ;)

########################################################################################

# Convertor sub routines ....
#
# History converting function

sub history_get($) {
	my $uin = shift;

	# we want to get all message block from ickle history file, so :
	$/ = "\n\n" ;

	if (-e "$home/.ickle/contacts/$uin.history") {
		open INFILE, "$home/.ickle/contacts/$uin.history" ;
	} else {
		$/ = "\n";
		return 1;
	}

	open OUTFILE, ">> $home/.tmp/ickle2kxicq/history" or die "Could not open temp file: $!\n";

	while ( defined(my $str=<INFILE>) ) {
		my @arr = split /\n/, $str;
		
		for (0 ... 5) {
			$arr[$_] =~ s/\w+:\s// ;
		}

		undef $str;
		for (5 ... (@arr - 1)) {
			$str .= " " . $arr[$_] ;
		}
		
		$str =~ s/\\/\\r\\n/g ;
		$str =~ s/^\s*// ;

		print OUTFILE "[Message $msg_num]\n";
		if ( $arr[1] =~ /\s*Sent\s*/ ) {
			print OUTFILE "Options=1 01 1 1\n";
		} else {
			print OUTFILE "Options=1 01 0 1\n";
		}
		print OUTFILE "Uin=$uin\nDateTime=" . time2mine(scalar $arr[0]) . "\n";
		print OUTFILE "Message=$str\n";

		$msg_num++;
	}

	close INFILE;
	close OUTFILE;

	$/ = "\n";
	return 1;
}


# Time converter
# Converting from ickle 5 kxicq time format ...

sub time2mine($) {
	my @t = localtime(shift);
	$t[4] += 1;
	$t[5] += 1900;
	return ($t[3] . '-' . $t[4] . '-' . $t[5] . ' ' . $t[2] . ':' . $t[1] );
}


# EOF
